
#include <sgl.h>
#include <sega_per.h>   //controller handling
#include <sega_bup.h>   //low level backup unit functions
#include "saturn.h"  //SatGetTime and SatInitTime Utility functions
#include "backup.h"  //Access to backup memoryw

void print_joystate(int x, int y, Uint16 iData)
{
   char acText[14];
   acText[0]  = iData&PER_DGT_U?' ':'u';
   acText[1]  = iData&PER_DGT_D?' ':'d';
   acText[2]  = iData&PER_DGT_R?' ':'r';
   acText[3]  = iData&PER_DGT_L?' ':'l';
   acText[4]  = iData&PER_DGT_A?' ':'A';
   acText[5]  = iData&PER_DGT_B?' ':'B';
   acText[6]  = iData&PER_DGT_C?' ':'C';
   acText[7]  = iData&PER_DGT_S?' ':'S';
   acText[8]  = iData&PER_DGT_X?' ':'X';
   acText[9]  = iData&PER_DGT_Y?' ':'Y';		 
   acText[10] = iData&PER_DGT_Z?' ':'Z';
   acText[11] = iData&PER_DGT_TR?' ':'R';
   acText[12] = iData&PER_DGT_TL?' ':'L';
   acText[13] = '\0';
   slPrint(acText,slLocate(x,y));
}

//size should be 300 bytes
//10 Levels: ordered by number of played seconds
typedef struct
{
   unsigned long adwScore1[10]; //40 Bytes
   unsigned long adwScore2[10]; //40 Bytes
   unsigned long adwScore3[10]; //40 Bytes
   unsigned char aacName1[10][3]; //30 Bytes
   unsigned char aacName2[10][3]; //30 Bytes
   unsigned char aacName3[10][3]; //30 Bytes
   unsigned char abDummy[90]; //unused
} tBackupData;

tBackupData tDefaultBackup =
{
    { 31,32,33,34,35,36,37,38,39,40 },
    { 21,22,23,24,25,26,27,28,29,30 },
    { 11,12,13,14,15,16,17,18,19,20 },
	{{ '1','1','1' }, { '2','2','2' }, { '3','3','3' }, { '4','4','4' }, { '5','5','5' }, 
	 { '6','6','6' }, { '7','7','7' }, { '8','8','8' }, { '9','9','9' }, { '0','0','0' }
	},
	{{ '1','1','1' }, { '2','2','2' }, { '3','3','3' }, { '4','4','4' }, { '5','5','5' }, 
	 { '6','6','6' }, { '7','7','7' }, { '8','8','8' }, { '9','9','9' }, { '0','0','0' }
	},
	{{ '1','1','1' }, { '2','2','2' }, { '3','3','3' }, { '4','4','4' }, { '5','5','5' }, 
	 { '6','6','6' }, { '7','7','7' }, { '8','8','8' }, { '9','9','9' }, { '0','0','0' }
	},
	{0}
};

tBackupData tBackup;

void display_highscores(unsigned char bLevel)
{
   unsigned char acText[200];
   slPrint("Highscores :",slLocate(1,20));
   sprintf(acText,"LV%02d: 1:%c%c%c/%03d 2:%c%c%c/%03d 3:%c%c%c/%03d"
           ,bLevel+1,
           tBackup.aacName1[bLevel][0],tBackup.aacName1[bLevel][1],tBackup.aacName1[bLevel][2],
           tBackup.adwScore1[bLevel],
           tBackup.aacName2[bLevel][0],tBackup.aacName2[bLevel][1],tBackup.aacName2[bLevel][2],
           tBackup.adwScore2[bLevel],
           tBackup.aacName3[bLevel][0],tBackup.aacName3[bLevel][1],tBackup.aacName3[bLevel][2],
           tBackup.adwScore3[bLevel]); 
   slPrint(acText,slLocate(1,21));
}

void enter_initials(unsigned char abName[3])
{
   int i;
   unsigned char bChar = 'A';
   unsigned char bOldUp = 1;
   unsigned char bOldDown = 1;
   unsigned char bOldStart = 1;
   Uint16 iData;
   char abText[200];
   slPrint("Enter Initials: ",slLocate(1,10));
   for (i=0;i<3;i++)
   {
	  while(-1)
	  {
         sprintf(abText,"%c",bChar);
         slPrint(abText,slLocate(17+i,10));
		 iData = Smpc_Peripheral[0].data;
		 print_joystate(1,5,iData);
	     //select character by moving pad up/down
		 if ((!(iData&PER_DGT_U)) && (bOldUp == 1))
		 {
		    if (bChar < 'Z')
			{
			   bChar++;
			}
		    bOldUp = 0;
		 }
		 if ((!(iData&PER_DGT_D)) && (bOldDown == 1))
		 {
		    if (bChar>'A')
			{
			   bChar--;
			}
		    bOldDown = 0;
		 }
         if ((bOldUp == 0) && (iData&PER_DGT_U))
		 {
		    bOldUp = 1;
		 }
         if ((bOldDown == 0) && (iData&PER_DGT_D))
		 {
		    bOldDown = 1;
  		 }
         //confirm character with "start"
         if ((!(iData&PER_DGT_S)) && (bOldStart == 1))
         {
		    bOldStart = 0;
			break;
		 }
		 if ((bOldStart == 0) && (iData&PER_DGT_S))
		 {
		    bOldStart = 1;
		 }
	  }
	  abName[i] = bChar;
   }
   //wait until "start" has been released
   while (!(Smpc_Peripheral[0].data&PER_DGT_S)) 
   { 
	  slSynch();
   }
   slPrint("                       ",slLocate(1,10));
}

void check_pad_connect(void)
{
   static unsigned char bOldState = 100;
   if (!Per_Connect1)
   {
      if (bOldState != 0)
	  {
         slPrint("No pad connected !!",slLocate(1,3)); 
		 bOldState = 0;
	  }
   }
   else
   {
      if (bOldState != 0)
	  {
         slPrint("Connected !!       ",slLocate(1,3));
		 bOldState = 1;
	  }
   }
}

void ss_main(void)
{
   #define MAX_LEVEL 9
   #define MIN_LEVEL 0
   char acText[200];
   long lStartTime,lActTime;
   unsigned char bPlay = 0;
   unsigned char bActLevel = 0;
   unsigned char bOldUp = 1;
   unsigned char bOldDown = 1;
   Uint16 iData;
   Uint16 iOldData = 0xFFFF;
   Uint16 iHelp;
   int i,iReturn;
   BupDir tDir;
   BupStat	sttb;
   unsigned char bBackupFree = 0;
   unsigned long dwPushes = 0;
   BupConfig conf[3];

   slInitSystem(TV_320x224,NULL,1);
   BackUpInit(conf); //init backup memory
   //check for free backup memory block
   BUP_Stat( 0, 6, &sttb );
   if (sttb.freeblock>6)
   {
      bBackupFree = 1;
   }
   strncpy( tDir.filename, "DRESFREQ", 9);
   strncpy( tDir.comment , "Frequency Scores", 17);
   tDir.language = BUP_ENGLISH;
   tDir.datasize = 300;
   tDir.date     = 0;
   iReturn = BUP_Read(0,"DRESFREQ",(unsigned char*)&tBackup);
   if (iReturn != 0)
   {
      memcpy((unsigned char *)&tBackup,(unsigned char*)&tDefaultBackup,sizeof(tBackupData));
   }
   else
   {
      bBackupFree = 1;
   }
   SatInitTime();
   display_highscores(bActLevel);
   while(1)
   {
      sprintf(acText,"Final score: %04d",dwPushes);
      slPrint(acText,slLocate(1,6));
      slPrint("Push \"Start\" to start.",slLocate(1,1));
      slPrint("Up/Down to select Level",slLocate(1,2));
      iData = Smpc_Peripheral[0].data;
	  while(iData&PER_DGT_S)
	  {
		 check_pad_connect();
		 iData = Smpc_Peripheral[0].data;
		 print_joystate(1,5,iData);
	     //select Level by moving pad up/down
		 if ((!(iData&PER_DGT_U)) && (bOldUp == 1))
		 {
		    if (bActLevel<MAX_LEVEL)
			{
			   bActLevel++;
			   display_highscores(bActLevel);
			}
		    bOldUp = 0;
		 }
		 if ((!(iData&PER_DGT_D)) && (bOldDown == 1))
		 {
		    if (bActLevel>MIN_LEVEL)
			{
			   bActLevel--;
			   display_highscores(bActLevel);
			}
		    bOldDown = 0;
		 }
         if ((bOldUp == 0) && (iData&PER_DGT_U))
		 {
		    bOldUp = 1;
		 }
         if ((bOldDown == 0) && (iData&PER_DGT_D))
		 {
		    bOldDown = 1;
		 }

		 sprintf(acText,"Actual level: %02d(%03dseconds)",bActLevel+1,(bActLevel+1)*2);
		 slPrint(acText,slLocate(1,4));
		 slSynch();
	  }
      slPrint("Go ...",slLocate(1,7));
      lStartTime = SatGetTime(); //-> in s
	  dwPushes = 0;
      do
	  {
	     lActTime = SatGetTime();
         sprintf(acText,"Time: %05d",(lActTime-lStartTime)/1000);
	     slPrint(acText,slLocate(1,8));
		 check_pad_connect();
		 SatGetTime();
         //info from Smpc_Peripheral is inverted for digital pads !
		 iData = Smpc_Peripheral[0].data;
		 print_joystate(1,5,iData);
		 //check how many buttons differ from old value:
		 SatGetTime();
         for (i=0;i<16;i++)
		 {
		    if (((iOldData & (1<<i)) > 0) && ((iData & (1<<i))==0))
			{
			   dwPushes++;
			}
		 }
		 iOldData = iData;
		 SatGetTime();
         sprintf(acText,"Pushes: %04d",dwPushes);
	     slPrint(acText,slLocate(1,9));
	  } while ((lActTime - ((bActLevel+1)*2000000)) < lStartTime);
	  if (dwPushes>tBackup.adwScore1[bActLevel])
	  {
	     tBackup.adwScore3[bActLevel] = tBackup.adwScore2[bActLevel];
	     tBackup.adwScore2[bActLevel] = tBackup.adwScore1[bActLevel];
	     tBackup.adwScore1[bActLevel] = dwPushes;
		 memcpy(tBackup.aacName3[bActLevel],tBackup.aacName2[bActLevel],3);
		 memcpy(tBackup.aacName2[bActLevel],tBackup.aacName1[bActLevel],3);
         enter_initials(tBackup.aacName1[bActLevel]);
		 display_highscores(bActLevel);
		 if (bBackupFree==1) BackUpWrite(0, &tDir, (unsigned char*)&tBackup,OFF);
	  }
	  else if (dwPushes>tBackup.adwScore2[bActLevel])
	  {
	     tBackup.adwScore3[bActLevel] = tBackup.adwScore2[bActLevel];
	     tBackup.adwScore2[bActLevel] = dwPushes;
		 memcpy(tBackup.aacName3[bActLevel],tBackup.aacName2[bActLevel],3);
         enter_initials(tBackup.aacName2[bActLevel]);
		 display_highscores(bActLevel);
		 if (bBackupFree==1) BackUpWrite(0, &tDir, (unsigned char*)&tBackup,OFF);
	  }
	  else if (dwPushes>tBackup.adwScore3[bActLevel])
	  {
	     tBackup.adwScore3[bActLevel] = dwPushes;
         enter_initials(tBackup.aacName3[bActLevel]);
		 display_highscores(bActLevel);
		 if (bBackupFree==1) BackUpWrite(0, &tDir, (unsigned char*)&tBackup,OFF);
	  }
      slSynch();
   }
}
